@@ -1,6 +1,4 @@
-use std::{
-    fmt, future::Future, io, marker, mem, net, pin::Pin, task::Context, task::Poll,
-};
+use std::{fmt, future::Future, io, marker, mem, net, pin::Pin, task::Context, task::Poll};
 
 use async_channel::{unbounded, Receiver};
 use async_oneshot as oneshot;
@@ -8,13 +6,16 @@ use futures_core::Stream;
 use log::{error, info};
 use socket2::{Domain, SockAddr, Socket, Type};
 
-use crate::rt::{net::TcpStream, spawn, System};
+use crate::io::Io;
+use crate::rt::{spawn, Signal, System};
+use crate::service::ServiceFactory;
 use crate::{time::sleep, time::Millis, util::join_all};
 
 use super::accept::{AcceptLoop, AcceptNotify, Command};
-use super::config::{ConfigWrapper, ConfiguredService, ServiceConfig, ServiceRuntime};
-use super::service::{Factory, InternalServiceFactory, StreamServiceFactory};
-use super::signals::{Signal, Signals};
+use super::config::{
+    Config, ConfigWrapper, ConfiguredService, ServiceConfig, ServiceRuntime,
+};
+use super::service::{Factory, InternalServiceFactory};
 use super::socket::Listener;
 use super::worker::{self, Worker, WorkerAvailability, WorkerClient};
 use super::{Server, ServerCommand, ServerStatus, Token};
@@ -186,15 +187,16 @@ impl ServerBuilder {
     }
 
     /// Add new service to the server.
-    pub fn bind<F, U, N: AsRef<str>>(
+    pub fn bind<F, U, N: AsRef<str>, R>(
         mut self,
         name: N,
         addr: U,
         factory: F,
     ) -> io::Result<Self>
     where
-        F: StreamServiceFactory<TcpStream>,
         U: net::ToSocketAddrs,
+        F: Fn(Config) -> R + Send + Clone + 'static,
+        R: ServiceFactory<Io>,
     {
         let sockets = bind_addr(addr, self.backlog)?;
 
@@ -206,22 +208,20 @@ impl ServerBuilder {
                 factory.clone(),
                 lst.local_addr()?,
             ));
-            self.sockets.push((
-                token,
-                name.as_ref().to_string(),
-                Listener::from_tcp(lst),
-            ));
+            self.sockets
+                .push((token, name.as_ref().to_string(), Listener::from_tcp(lst)));
         }
         Ok(self)
     }
 
     #[cfg(all(unix))]
     /// Add new unix domain service to the server.
-    pub fn bind_uds<F, U, N>(self, name: N, addr: U, factory: F) -> io::Result<Self>
+    pub fn bind_uds<F, U, N, R>(self, name: N, addr: U, factory: F) -> io::Result<Self>
     where
-        F: StreamServiceFactory<crate::rt::net::UnixStream>,
         N: AsRef<str>,
         U: AsRef<std::path::Path>,
+        F: Fn(Config) -> R + Send + Clone + 'static,
+        R: ServiceFactory<Io>,
     {
         use std::os::unix::net::UnixListener;
 
@@ -242,14 +242,15 @@ impl ServerBuilder {
     /// Add new unix domain service to the server.
     /// Useful when running as a systemd service and
     /// a socket FD can be acquired using the systemd crate.
-    pub fn listen_uds<F, N: AsRef<str>>(
+    pub fn listen_uds<F, N: AsRef<str>, R>(
         mut self,
         name: N,
         lst: std::os::unix::net::UnixListener,
         factory: F,
     ) -> io::Result<Self>
     where
-        F: StreamServiceFactory<crate::rt::net::UnixStream>,
+        F: Fn(Config) -> R + Send + Clone + 'static,
+        R: ServiceFactory<Io>,
     {
         use std::net::{IpAddr, Ipv4Addr, SocketAddr};
         let token = self.token.next();
@@ -266,14 +267,15 @@ impl ServerBuilder {
     }
 
     /// Add new service to the server.
-    pub fn listen<F, N: AsRef<str>>(
+    pub fn listen<F, N: AsRef<str>, R>(
         mut self,
         name: N,
         lst: net::TcpListener,
         factory: F,
     ) -> io::Result<Self>
     where
-        F: StreamServiceFactory<TcpStream>,
+        F: Fn(Config) -> R + Send + Clone + 'static,
+        R: ServiceFactory<Io>,
     {
         let token = self.token.next();
         self.services.push(Factory::create(
@@ -287,11 +289,6 @@ impl ServerBuilder {
         Ok(self)
     }
 
-    #[doc(hidden)]
-    pub fn start(self) -> Server {
-        self.run()
-    }
-
     /// Starts processing incoming connections and return server controller.
     pub fn run(mut self) -> Server {
         if self.sockets.is_empty() {
@@ -321,7 +318,7 @@ impl ServerBuilder {
 
             // handle signals
             if !self.no_signals {
-                spawn(Signals::new(self.server.clone()));
+                spawn(signals(self.server.clone()));
             }
 
             // start http server actor
@@ -478,6 +475,21 @@ impl Future for ServerBuilder {
     }
 }
 
+async fn signals(srv: Server) {
+    loop {
+        if let Some(rx) = crate::rt::signal() {
+            if let Ok(sig) = rx.await {
+                srv.signal(sig);
+            } else {
+                return;
+            }
+        } else {
+            log::info!("Signals are not supported by current runtime");
+            return;
+        }
+    }
+}
+
 pub(super) fn bind_addr<S: net::ToSocketAddrs>(
     addr: S,
     backlog: i32,
@@ -532,51 +544,6 @@ pub(crate) fn create_tcp_listener(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::server::{signals, Server, TestServer};
-    use crate::service::fn_service;
-
-    #[cfg(unix)]
-    #[crate::rt_test]
-    async fn test_signals() {
-        use std::{net, sync::mpsc, thread};
-
-        fn start(tx: mpsc::Sender<(Server, net::SocketAddr)>) -> thread::JoinHandle<()> {
-            thread::spawn(move || {
-                let mut sys = crate::rt::System::new("test");
-                let addr = TestServer::unused_addr();
-                let srv = sys.exec(|| {
-                    crate::server::build()
-                        .workers(1)
-                        .disable_signals()
-                        .bind("test", addr, move || {
-                            fn_service(|_| async { Ok::<_, ()>(()) })
-                        })
-                        .unwrap()
-                        .start()
-                });
-                let _ = tx.send((srv, addr));
-                let _ = sys.run();
-            })
-        }
-
-        for sig in &[
-            signals::Signal::Int,
-            signals::Signal::Term,
-            signals::Signal::Quit,
-        ] {
-            let (tx, rx) = mpsc::channel();
-            let h = start(tx);
-            let (srv, addr) = rx.recv().unwrap();
-
-            crate::time::sleep(Millis(300)).await;
-            assert!(net::TcpStream::connect(addr).is_ok());
-
-            srv.signal(*sig);
-            crate::time::sleep(Millis(300)).await;
-            assert!(net::TcpStream::connect(addr).is_err());
-            let _ = h.join();
-        }
-    }
 
     #[test]
     fn test_bind_addr() {
