@@ -1,121 +1,113 @@
 //! Contains `Variant` service and related types and functions.
-use std::{future::Future, pin::Pin, task::Context, task::Poll};
+use std::{future::Future, marker::PhantomData, pin::Pin, task::Context, task::Poll};
 
 use crate::service::{IntoServiceFactory, Service, ServiceFactory};
 
 /// Construct `Variant` service factory.
 ///
 /// Variant service allow to combine multiple different services into a single service.
-pub fn variant<A: ServiceFactory>(factory: A) -> Variant<A> {
-    Variant { factory }
+pub fn variant<V1: ServiceFactory<V1R, V1C>, V1R, V1C>(
+    factory: V1,
+) -> Variant<V1, V1R, V1C> {
+    Variant {
+        factory,
+        _t: PhantomData,
+    }
 }
 
 /// Combine multiple different service types into a single service.
-pub struct Variant<A> {
+pub struct Variant<A, AR, AC> {
     factory: A,
+    _t: PhantomData<(AR, AC)>,
 }
 
-impl<A> Variant<A>
+impl<A, AR, AC> Variant<A, AR, AC>
 where
-    A: ServiceFactory,
-    A::Config: Clone,
+    A: ServiceFactory<AR, AC>,
+    AC: Clone,
 {
     /// Convert to a Variant with two request types
-    pub fn and<B, F>(self, factory: F) -> VariantFactory2<A, B>
+    pub fn v2<B, BR, F>(self, factory: F) -> VariantFactory2<A, AC, B, AR, BR>
     where
         B: ServiceFactory<
-            Config = A::Config,
+            BR,
+            AC,
             Response = A::Response,
             Error = A::Error,
             InitError = A::InitError,
         >,
-        F: IntoServiceFactory<B>,
+        F: IntoServiceFactory<B, BR, AC>,
     {
         VariantFactory2 {
-            A: self.factory,
-            V2: factory.into_factory(),
-        }
-    }
-
-    /// Convert to a Variant with two request types
-    pub fn v2<B, F>(self, factory: F) -> VariantFactory2<A, B>
-    where
-        B: ServiceFactory<
-            Config = A::Config,
-            Response = A::Response,
-            Error = A::Error,
-            InitError = A::InitError,
-        >,
-        F: IntoServiceFactory<B>,
-    {
-        VariantFactory2 {
-            A: self.factory,
+            V1: self.factory,
             V2: factory.into_factory(),
+            _t: PhantomData,
         }
     }
 }
 
-macro_rules! variant_impl_and ({$fac1_type:ident, $fac2_type:ident, $name:ident, $m_name:ident, ($($T:ident),+)} => {
+macro_rules! variant_impl_and ({$fac1_type:ident, $fac2_type:ident, $name:ident, $r_name:ident, $m_name:ident, ($($T:ident),+), ($($R:ident),+)} => {
 
-    impl<V1, $($T),+> $fac1_type<V1, $($T),+>
+    #[allow(non_snake_case)]
+    impl<V1, V1C, $($T,)+ V1R, $($R,)+> $fac1_type<V1, V1C, $($T,)+ V1R, $($R,)+>
         where
-            V1: ServiceFactory,
-            V1::Config: Clone,
+            V1: ServiceFactory<V1R, V1C>,
+            V1C: Clone,
         {
             /// Convert to a Variant with more request types
-            pub fn $m_name<$name, F>(self, factory: F) -> $fac2_type<V1, $($T,)+ $name>
-            where $name: ServiceFactory<
-                    Config = V1::Config,
+            pub fn $m_name<$name, $r_name, F>(self, factory: F) -> $fac2_type<V1, V1C, $($T,)+ $name, V1R, $($R,)+ $r_name>
+            where $name: ServiceFactory<$r_name, V1C,
                     Response = V1::Response,
                     Error = V1::Error,
                     InitError = V1::InitError>,
-                F: IntoServiceFactory<$name>,
+                F: IntoServiceFactory<$name, $r_name, V1C>,
             {
                 $fac2_type {
-                    A: self.A,
+                    V1: self.V1,
                     $($T: self.$T,)+
                     $name: factory.into_factory(),
+                    _t: PhantomData
                 }
             }
     }
 });
 
-macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident, $fac_type:ident, $(($n:tt, $T:ident)),+} => {
+macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident, $fac_type:ident, $(($n:tt, $T:ident, $R:ident)),+} => {
 
     #[allow(non_snake_case)]
-    pub enum $enum_type<V1, $($T),+> {
-        V1(V1),
-        $($T($T),)+
+    pub enum $enum_type<V1R, $($R),+> {
+        V1(V1R),
+        $($T($R),)+
     }
 
     #[allow(non_snake_case)]
-    pub struct $srv_type<V1, $($T),+> {
-        a: V1,
+    pub struct $srv_type<V1, $($T,)+ V1R, $($R,)+> {
+        V1: V1,
         $($T: $T,)+
+        _t: PhantomData<(V1R, $($R),+)>,
     }
 
-    impl<V1: Clone, $($T: Clone),+> Clone for $srv_type<V1, $($T),+> {
+    impl<V1: Clone, $($T: Clone,)+ V1R, $($R,)+> Clone for $srv_type<V1, $($T,)+ V1R, $($R,)+> {
         fn clone(&self) -> Self {
             Self {
-                a: self.a.clone(),
+                _t: PhantomData,
+                V1: self.V1.clone(),
                 $($T: self.$T.clone(),)+
             }
         }
     }
 
-    impl<V1, $($T),+> Service for $srv_type<V1, $($T),+>
+    impl<V1, $($T,)+ V1R, $($R,)+> Service<$enum_type<V1R, $($R,)+>> for $srv_type<V1, $($T,)+ V1R, $($R,)+>
     where
-        V1: Service,
-        $($T: Service<Response = V1::Response, Error = V1::Error>),+
+        V1: Service<V1R>,
+        $($T: Service<$R, Response = V1::Response, Error = V1::Error>),+
     {
-        type Request = $enum_type<V1::Request, $($T::Request),+>;
         type Response = V1::Response;
         type Error = V1::Error;
         type Future = $mod_name::ServiceResponse<V1::Future, $($T::Future),+>;
 
-        #[inline]
         fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-            let mut ready = self.a.poll_ready(cx)?.is_ready();
+            let mut ready = self.V1.poll_ready(cx)?.is_ready();
             $(ready = self.$T.poll_ready(cx)?.is_ready() && ready;)+
 
             if ready {
@@ -125,62 +117,60 @@ macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident,
             }
         }
 
-        #[inline]
         fn poll_shutdown(&self, cx: &mut Context<'_>, is_error: bool) -> Poll<()> {
-            let mut ready = self.a.poll_shutdown(cx, is_error).is_ready();
-        $(ready = self.$T.poll_shutdown(cx, is_error).is_ready() && ready;)+
+            let mut ready = self.V1.poll_shutdown(cx, is_error).is_ready();
+            $(ready = self.$T.poll_shutdown(cx, is_error).is_ready() && ready;)+
 
-        if ready {
-            Poll::Ready(())
-        } else {
-            Poll::Pending
+            if ready {
+                Poll::Ready(())
+            } else {
+                Poll::Pending
+            }
         }
-    }
 
-        #[inline]
-        fn call(&self, req: Self::Request) -> Self::Future {
+        fn call(&self, req: $enum_type<V1R, $($R,)+>) -> Self::Future {
             match req {
-                $enum_type::V1(req) => $mod_name::ServiceResponse::V1 { fut: self.a.call(req) },
+                $enum_type::V1(req) => $mod_name::ServiceResponse::V1 { fut: self.V1.call(req) },
                 $($enum_type::$T(req) => $mod_name::ServiceResponse::$T { fut: self.$T.call(req) },)+
             }
         }
     }
 
     #[allow(non_snake_case)]
-    pub struct $fac_type<V1, $($T),+> {
-        A: V1,
+    pub struct $fac_type<V1, V1C, $($T,)+ V1R, $($R,)+> {
+        V1: V1,
         $($T: $T,)+
+        _t: PhantomData<(V1C, V1R, $($R,)+)>,
     }
 
-    impl<V1: Clone, $($T: Clone),+> Clone for $fac_type<V1, $($T),+> {
+    impl<V1: Clone, V1C, $($T: Clone,)+ V1R, $($R,)+> Clone for $fac_type<V1, V1C, $($T,)+ V1R, $($R,)+> {
         fn clone(&self) -> Self {
             Self {
-                A: self.A.clone(),
+                _t: PhantomData,
+                V1: self.V1.clone(),
                 $($T: self.$T.clone(),)+
             }
         }
     }
 
-    impl<V1, $($T),+> ServiceFactory for $fac_type<V1, $($T),+>
+    impl<V1, V1C, $($T,)+ V1R, $($R,)+> ServiceFactory<$enum_type<V1R, $($R),+>, V1C> for $fac_type<V1, V1C, $($T,)+ V1R, $($R,)+>
     where
-        V1: ServiceFactory,
-        V1::Config: Clone,
-        $($T: ServiceFactory<Config = V1::Config, Response = V1::Response, Error = V1::Error, InitError = V1::InitError>),+
+        V1: ServiceFactory<V1R, V1C>,
+        V1C: Clone,
+        $($T: ServiceFactory<$R, V1C, Response = V1::Response, Error = V1::Error, InitError = V1::InitError>),+
     {
-        type Request = $enum_type<V1::Request, $($T::Request),+>;
         type Response = V1::Response;
         type Error = V1::Error;
-        type Config = V1::Config;
         type InitError = V1::InitError;
-        type Service = $srv_type<V1::Service, $($T::Service,)+>;
-        type Future = $mod_name::ServiceFactoryResponse<V1, $($T,)+>;
+        type Service = $srv_type<V1::Service, $($T::Service,)+ V1R, $($R,)+>;
+        type Future = $mod_name::ServiceFactoryResponse<V1, V1C, $($T,)+ V1R, $($R,)+>;
 
-        fn new_service(&self, cfg: Self::Config) -> Self::Future {
+        fn new_service(&self, cfg: V1C) -> Self::Future {
             $mod_name::ServiceFactoryResponse {
-                a: None,
+                V1: None,
                 items: Default::default(),
                 $($T: self.$T.new_service(cfg.clone()),)+
-                a_fut: self.A.new_service(cfg),
+                V1_fut: self.V1.new_service(cfg),
             }
         }
     }
@@ -192,18 +182,18 @@ macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident,
 
         pin_project_lite::pin_project! {
             #[project = ServiceResponseProject]
-            pub enum ServiceResponse<A: Future, $($T: Future),+> {
-                V1{ #[pin] fut: A },
+            pub enum ServiceResponse<V1: Future, $($T: Future),+> {
+                V1{ #[pin] fut: V1 },
                 $($T{ #[pin] fut: $T },)+
             }
         }
 
-        impl<A, $($T),+> Future for ServiceResponse<A, $($T),+>
+        impl<V1, $($T),+> Future for ServiceResponse<V1, $($T),+>
         where
-            A: Future,
-            $($T: Future<Output = A::Output>),+
+            V1: Future,
+            $($T: Future<Output = V1::Output>),+
         {
-            type Output = A::Output;
+            type Output = V1::Output;
 
             fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
                 match self.project() {
@@ -216,29 +206,29 @@ macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident,
 
         pin_project_lite::pin_project! {
             #[doc(hidden)]
-            pub struct ServiceFactoryResponse<A: ServiceFactory, $($T: ServiceFactory),+> {
-                pub(super) a: Option<A::Service>,
+            pub struct ServiceFactoryResponse<V1: ServiceFactory<V1R, V1C>, V1C, $($T: ServiceFactory<$R, V1C>,)+ V1R, $($R,)+> {
+                pub(super) V1: Option<V1::Service>,
                 pub(super) items: ($(Option<$T::Service>,)+),
-                #[pin] pub(super) a_fut: A::Future,
+                #[pin] pub(super) V1_fut: V1::Future,
                 $(#[pin] pub(super) $T: $T::Future),+
             }
         }
 
-        impl<A, $($T),+> Future for ServiceFactoryResponse<A, $($T),+>
+        impl<V1, V1C, $($T,)+ V1R, $($R,)+> Future for ServiceFactoryResponse<V1, V1C, $($T,)+ V1R, $($R,)+>
         where
-            A: ServiceFactory,
-        $($T: ServiceFactory<Response = A::Response, Error = A::Error, InitError = A::InitError,>),+
+            V1: ServiceFactory<V1R, V1C>,
+        $($T: ServiceFactory<$R, V1C, Response = V1::Response, Error = V1::Error, InitError = V1::InitError,>),+
         {
-            type Output = Result<$srv_type<A::Service, $($T::Service),+>, A::InitError>;
+            type Output = Result<$srv_type<V1::Service, $($T::Service,)+ V1R, $($R),+>, V1::InitError>;
 
             fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
                 let this = self.project();
                 let mut ready = true;
 
-                if this.a.is_none() {
-                    match this.a_fut.poll(cx) {
+                if this.V1.is_none() {
+                    match this.V1_fut.poll(cx) {
                         Poll::Ready(Ok(item)) => {
-                            *this.a = Some(item);
+                            *this.V1 = Some(item);
                         }
                         Poll::Pending => ready = false,
                         Poll::Ready(Err(e)) => return Poll::Ready(Err(e.into())),
@@ -259,8 +249,9 @@ macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident,
 
                     if ready {
                         Poll::Ready(Ok($srv_type {
-                            a: this.a.take().unwrap(),
+                            V1: this.V1.take().unwrap(),
                             $($T: this.items.$n.take().unwrap(),)+
+                            _t: PhantomData
                         }))
                     } else {
                         Poll::Pending
@@ -272,33 +263,32 @@ macro_rules! variant_impl ({$mod_name:ident, $enum_type:ident, $srv_type:ident,
 });
 
 #[rustfmt::skip]
-variant_impl!(v2, Variant2, VariantService2, VariantFactory2, (0, V2));
+variant_impl!(v2, Variant2, VariantService2, VariantFactory2, (0, V2, V2R));
 #[rustfmt::skip]
-variant_impl!(v3, Variant3, VariantService3, VariantFactory3, (0, V2), (1, V3));
+variant_impl!(v3, Variant3, VariantService3, VariantFactory3, (0, V2, V2R), (1, V3, V3R));
 #[rustfmt::skip]
-variant_impl!(v4, Variant4, VariantService4, VariantFactory4, (0, V2), (1, V3), (2, V4));
+variant_impl!(v4, Variant4, VariantService4, VariantFactory4, (0, V2, V2R), (1, V3, V3R), (2, V4, V4R));
 #[rustfmt::skip]
-variant_impl!(v5, Variant5, VariantService5, VariantFactory5, (0, V2), (1, V3), (2, V4), (3, V5));
+variant_impl!(v5, Variant5, VariantService5, VariantFactory5, (0, V2, V2R), (1, V3, V3R), (2, V4, V4R), (3, V5, V5R));
 #[rustfmt::skip]
-variant_impl!(v6, Variant6, VariantService6, VariantFactory6, (0, V2), (1, V3), (2, V4), (3, V5), (4, V6));
+variant_impl!(v6, Variant6, VariantService6, VariantFactory6, (0, V2, V2R), (1, V3, V3R), (2, V4, V4R), (3, V5, V5R), (4, V6, V6R));
 #[rustfmt::skip]
-variant_impl!(v7, Variant7, VariantService7, VariantFactory7, (0, V2), (1, V3), (2, V4), (3, V5), (4, V6), (5, V7));
+variant_impl!(v7, Variant7, VariantService7, VariantFactory7, (0, V2, V2R), (1, V3, V3R), (2, V4, V4R), (3, V5, V5R), (4, V6, V6R), (5, V7, V7R));
 #[rustfmt::skip]
-variant_impl!(v8, Variant8, VariantService8, VariantFactory8, (0, V2), (1, V3), (2, V4), (3, V5), (4, V6), (5, V7), (6, V8));
+variant_impl!(v8, Variant8, VariantService8, VariantFactory8, (0, V2, V2R), (1, V3, V3R), (2, V4, V4R), (3, V5, V5R), (4, V6, V6R), (5, V7, V7R), (6, V8, V8R));
 
-variant_impl_and!(VariantFactory2, VariantFactory3, V3, v3, (V2));
-variant_impl_and!(VariantFactory3, VariantFactory4, V4, v4, (V2, V3));
-variant_impl_and!(VariantFactory4, VariantFactory5, V5, v5, (V2, V3, V4));
-variant_impl_and!(VariantFactory5, VariantFactory6, V6, v6, (V2, V3, V4, V5));
-variant_impl_and!(
-    VariantFactory6,
-    VariantFactory7,
-    V7,
-    v7,
-    (V2, V3, V4, V5, V6)
-);
 #[rustfmt::skip]
-variant_impl_and!(VariantFactory7, VariantFactory8, V8, v8, (V2, V3, V4, V5, V6, V7));
+variant_impl_and!(VariantFactory2, VariantFactory3, V3, V3R, v3, (V2), (V2R));
+#[rustfmt::skip]
+variant_impl_and!(VariantFactory3, VariantFactory4, V4, V4R, v4, (V2, V3), (V2R, V3R));
+#[rustfmt::skip]
+variant_impl_and!(VariantFactory4, VariantFactory5, V5, V5R, v5, (V2, V3, V4), (V2R, V3R, V4R));
+#[rustfmt::skip]
+variant_impl_and!(VariantFactory5, VariantFactory6, V6, V6R, v6, (V2, V3, V4, V5), (V2R, V3R, V4R, V5R));
+#[rustfmt::skip]
+variant_impl_and!(VariantFactory6, VariantFactory7, V7, V7R, v7, (V2, V3, V4, V5, V6), (V2R, V3R, V4R, V5R, V6R));
+#[rustfmt::skip]
+variant_impl_and!(VariantFactory7, VariantFactory8, V8, V8R, v8, (V2, V3, V4, V5, V6, V7), (V2R, V3R, V4R, V5R, V6R, V7R));
 
 #[cfg(test)]
 mod tests {
@@ -311,8 +301,7 @@ mod tests {
     #[derive(Clone)]
     struct Srv1;
 
-    impl Service for Srv1 {
-        type Request = ();
+    impl Service<()> for Srv1 {
         type Response = usize;
         type Error = ();
         type Future = Ready<usize, ()>;
@@ -333,8 +322,7 @@ mod tests {
     #[derive(Clone)]
     struct Srv2;
 
-    impl Service for Srv2 {
-        type Request = ();
+    impl Service<()> for Srv2 {
         type Response = usize;
         type Error = ();
         type Future = Ready<usize, ()>;
