@@ -28,7 +28,7 @@ type FnDataFactory =
 /// for building application instances.
 pub struct App<M, F, Err: ErrorRenderer = DefaultError> {
     middleware: M,
-    filter: PipelineFactory<F>,
+    filter: PipelineFactory<F, WebRequest<Err>>,
     services: Vec<Box<dyn AppServiceFactory<Err>>>,
     default: Option<Rc<HttpNewService<Err>>>,
     data: Vec<Box<dyn DataFactory>>,
@@ -78,8 +78,7 @@ impl<Err: ErrorRenderer> App<Identity, Filter<Err>, Err> {
 impl<M, T, Err> App<M, T, Err>
 where
     T: ServiceFactory<
-        Config = (),
-        Request = WebRequest<Err>,
+        WebRequest<Err>,
         Response = WebRequest<Err>,
         Error = Err::Container,
         InitError = (),
@@ -280,13 +279,9 @@ where
     /// ```
     pub fn default_service<F, U>(mut self, f: F) -> Self
     where
-        F: IntoServiceFactory<U>,
-        U: ServiceFactory<
-                Config = (),
-                Request = WebRequest<Err>,
-                Response = WebResponse,
-                Error = Err::Container,
-            > + 'static,
+        F: IntoServiceFactory<U, WebRequest<Err>>,
+        U: ServiceFactory<WebRequest<Err>, Response = WebResponse, Error = Err::Container>
+            + 'static,
         U::InitError: fmt::Debug,
     {
         // create and configure default resource
@@ -361,8 +356,7 @@ where
     ) -> App<
         M,
         impl ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
+            WebRequest<Err>,
             Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
@@ -371,13 +365,12 @@ where
     >
     where
         S: ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
+            WebRequest<Err>,
             Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         >,
-        U: IntoServiceFactory<S>,
+        U: IntoServiceFactory<S, WebRequest<Err>>,
     {
         App {
             filter: self.filter.and_then(filter.into_factory()),
@@ -446,14 +439,9 @@ where
 impl<M, F, Err> App<M, F, Err>
 where
     M: Transform<AppService<F::Service, Err>> + 'static,
-    M::Service: Service<
-        Request = WebRequest<Err>,
-        Response = WebResponse,
-        Error = Err::Container,
-    >,
+    M::Service: Service<WebRequest<Err>, Response = WebResponse, Error = Err::Container>,
     F: ServiceFactory<
-        Config = (),
-        Request = WebRequest<Err>,
+        WebRequest<Err>,
         Response = WebRequest<Err>,
         Error = Err::Container,
         InitError = (),
@@ -468,12 +456,12 @@ where
     ///
     /// #[ntex::main]
     /// async fn main() -> std::io::Result<()> {
-    ///     server::build().bind("http", "127.0.0.1:0", ||
+    ///     server::build().bind("http", "127.0.0.1:0", |_|
     ///         http::HttpService::build().finish(
     ///             web::App::new()
     ///                 .route("/index.html", web::get().to(|| async { "hello_world" }))
     ///                 .finish()
-    ///         ).tcp()
+    ///         )
     ///     )?
     ///     .run()
     ///     .await
@@ -482,8 +470,7 @@ where
     pub fn finish(
         self,
     ) -> impl ServiceFactory<
-        Config = (),
-        Request = Request,
+        Request,
         Response = WebResponse,
         Error = Err::Container,
         InitError = (),
@@ -498,12 +485,12 @@ where
     ///
     /// #[ntex::main]
     /// async fn main() -> std::io::Result<()> {
-    ///     server::build().bind("http", "127.0.0.1:0", ||
+    ///     server::build().bind("http", "127.0.0.1:0", |_|
     ///         http::HttpService::build().finish(
     ///             web::App::new()
     ///                 .route("/index.html", web::get().to(|| async { "hello_world" }))
     ///                 .with_config(web::dev::AppConfig::default())
-    ///         ).tcp()
+    ///         )
     ///     )?
     ///     .run()
     ///     .await
@@ -513,8 +500,7 @@ where
         self,
         cfg: AppConfig,
     ) -> impl ServiceFactory<
-        Config = (),
-        Request = Request,
+        Request,
         Response = WebResponse,
         Error = Err::Container,
         InitError = (),
@@ -523,17 +509,13 @@ where
     }
 }
 
-impl<M, F, Err> IntoServiceFactory<AppFactory<M, F, Err>> for App<M, F, Err>
+impl<M, F, Err> IntoServiceFactory<AppFactory<M, F, Err>, Request, AppConfig>
+    for App<M, F, Err>
 where
     M: Transform<AppService<F::Service, Err>> + 'static,
-    M::Service: Service<
-        Request = WebRequest<Err>,
-        Response = WebResponse,
-        Error = Err::Container,
-    >,
+    M::Service: Service<WebRequest<Err>, Response = WebResponse, Error = Err::Container>,
     F: ServiceFactory<
-        Config = (),
-        Request = WebRequest<Err>,
+        WebRequest<Err>,
         Response = WebRequest<Err>,
         Error = Err::Container,
         InitError = (),
@@ -587,9 +569,7 @@ impl<Err: ErrorRenderer> Filter<Err> {
     }
 }
 
-impl<Err: ErrorRenderer> ServiceFactory for Filter<Err> {
-    type Config = ();
-    type Request = WebRequest<Err>;
+impl<Err: ErrorRenderer> ServiceFactory<WebRequest<Err>> for Filter<Err> {
     type Response = WebRequest<Err>;
     type Error = Err::Container;
     type InitError = ();
@@ -602,22 +582,18 @@ impl<Err: ErrorRenderer> ServiceFactory for Filter<Err> {
     }
 }
 
-impl<Err: ErrorRenderer> Service for Filter<Err> {
-    type Request = WebRequest<Err>;
+impl<Err: ErrorRenderer> Service<WebRequest<Err>> for Filter<Err> {
     type Response = WebRequest<Err>;
     type Error = Err::Container;
     type Future = Ready<WebRequest<Err>, Err::Container>;
 
     #[inline]
-    fn poll_ready(
-        &self,
-        _: &mut task::Context<'_>,
-    ) -> task::Poll<Result<(), Self::Error>> {
+    fn poll_ready(&self, _: &mut task::Context<'_>) -> task::Poll<Result<(), Self::Error>> {
         task::Poll::Ready(Ok(()))
     }
 
     #[inline]
-    fn call(&self, req: Self::Request) -> Self::Future {
+    fn call(&self, req: WebRequest<Err>) -> Self::Future {
         Ready::Ok(req)
     }
 }
@@ -631,15 +607,14 @@ mod tests {
     use crate::util::{Bytes, Ready};
     use crate::web::test::{call_service, init_service, read_body, TestRequest};
     use crate::web::{
-        self, middleware::DefaultHeaders, request::WebRequest, DefaultError,
-        HttpRequest, HttpResponse,
+        self, middleware::DefaultHeaders, request::WebRequest, DefaultError, HttpRequest,
+        HttpResponse,
     };
 
     #[crate::rt_test]
     async fn test_default_resource() {
         let srv = init_service(
-            App::new()
-                .service(web::resource("/test").to(|| async { HttpResponse::Ok() })),
+            App::new().service(web::resource("/test").to(|| async { HttpResponse::Ok() })),
         )
         .await;
         let req = TestRequest::with_uri("/test").to_request();
