@@ -2,11 +2,10 @@ use std::sync::atomic::{AtomicUsize, Ordering::Relaxed};
 use std::sync::{mpsc, Arc};
 use std::{io, io::Read, net, thread, time};
 
-use futures::future::{lazy, ok, FutureExt};
-use futures::SinkExt;
+use futures::future::{ok, FutureExt};
 
-use ntex::codec::{BytesCodec, Framed};
-use ntex::rt::net::TcpStream;
+use ntex::codec::BytesCodec;
+use ntex::io::Io;
 use ntex::server::{Server, TestServer};
 use ntex::service::fn_service;
 use ntex::util::{Bytes, Ready};
@@ -17,14 +16,14 @@ fn test_bind() {
     let (tx, rx) = mpsc::channel();
 
     let h = thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
-        let srv = sys.exec(|| {
+        let sys = ntex::rt::System::new("test");
+        let srv = sys.exec(move || {
             Server::build()
                 .workers(1)
                 .disable_signals()
-                .bind("test", addr, move || fn_service(|_| ok::<_, ()>(())))
+                .bind("test", addr, move |_| fn_service(|_| ok::<_, ()>(())))
                 .unwrap()
-                .start()
+                .run()
         });
         let _ = tx.send((srv, ntex::rt::System::current()));
         let _ = sys.run();
@@ -43,15 +42,15 @@ fn test_listen() {
     let (tx, rx) = mpsc::channel();
 
     let h = thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
         let lst = net::TcpListener::bind(addr).unwrap();
-        sys.exec(|| {
+        sys.exec(move || {
             Server::build()
                 .disable_signals()
                 .workers(1)
-                .listen("test", lst, move || fn_service(|_| ok::<_, ()>(())))
+                .listen("test", lst, move |_| fn_service(|_| ok::<_, ()>(())))
                 .unwrap()
-                .start()
+                .run()
         });
         let _ = tx.send(ntex::rt::System::current());
         let _ = sys.run();
@@ -66,25 +65,26 @@ fn test_listen() {
 
 #[test]
 #[cfg(unix)]
-fn test_start() {
+fn test_run() {
     let addr = TestServer::unused_addr();
     let (tx, rx) = mpsc::channel();
 
     let h = thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
-        let srv = sys.exec(|| {
+        let sys = ntex::rt::System::new("test");
+        let srv = sys.exec(move || {
             Server::build()
                 .backlog(100)
                 .disable_signals()
-                .bind("test", addr, move || {
-                    fn_service(|io: TcpStream| async move {
-                        let mut f = Framed::new(io, BytesCodec);
-                        f.send(Bytes::from_static(b"test")).await.unwrap();
+                .bind("test", addr, move |_| {
+                    fn_service(|io: Io| async move {
+                        io.send(Bytes::from_static(b"test"), &BytesCodec)
+                            .await
+                            .unwrap();
                         Ok::<_, ()>(())
                     })
                 })
                 .unwrap()
-                .start()
+                .run()
         });
 
         let _ = tx.send((srv, ntex::rt::System::current()));
@@ -128,109 +128,6 @@ fn test_start() {
     let _ = h.join();
 }
 
-#[test]
-fn test_configure() {
-    let addr1 = TestServer::unused_addr();
-    let addr2 = TestServer::unused_addr();
-    let addr3 = TestServer::unused_addr();
-    let (tx, rx) = mpsc::channel();
-    let num = Arc::new(AtomicUsize::new(0));
-    let num2 = num.clone();
-
-    let h = thread::spawn(move || {
-        let num = num2.clone();
-        let mut sys = ntex::rt::System::new("test");
-        let srv = sys.exec(|| {
-            Server::build()
-                .disable_signals()
-                .configure(move |cfg| {
-                    let num = num.clone();
-                    let lst = net::TcpListener::bind(addr3).unwrap();
-                    cfg.bind("addr1", addr1)
-                        .unwrap()
-                        .bind("addr2", addr2)
-                        .unwrap()
-                        .listen("addr3", lst)
-                        .apply(move |rt| {
-                            let num = num.clone();
-                            rt.service("addr1", fn_service(|_| ok::<_, ()>(())));
-                            rt.service("addr3", fn_service(|_| ok::<_, ()>(())));
-                            rt.on_start(lazy(move |_| {
-                                let _ = num.fetch_add(1, Relaxed);
-                            }))
-                        })
-                })
-                .unwrap()
-                .workers(1)
-                .start()
-        });
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
-    });
-    let (_, sys) = rx.recv().unwrap();
-    thread::sleep(time::Duration::from_millis(500));
-
-    assert!(net::TcpStream::connect(addr1).is_ok());
-    assert!(net::TcpStream::connect(addr2).is_ok());
-    assert!(net::TcpStream::connect(addr3).is_ok());
-    assert_eq!(num.load(Relaxed), 1);
-    sys.stop();
-    let _ = h.join();
-}
-
-#[test]
-fn test_configure_async() {
-    let addr1 = TestServer::unused_addr();
-    let addr2 = TestServer::unused_addr();
-    let addr3 = TestServer::unused_addr();
-    let (tx, rx) = mpsc::channel();
-    let num = Arc::new(AtomicUsize::new(0));
-    let num2 = num.clone();
-
-    let h = thread::spawn(move || {
-        let num = num2.clone();
-        let mut sys = ntex::rt::System::new("test");
-        let srv = sys.exec(|| {
-            Server::build()
-                .disable_signals()
-                .configure(move |cfg| {
-                    let num = num.clone();
-                    let lst = net::TcpListener::bind(addr3).unwrap();
-                    cfg.bind("addr1", addr1)
-                        .unwrap()
-                        .bind("addr2", addr2)
-                        .unwrap()
-                        .listen("addr3", lst)
-                        .apply_async(move |rt| {
-                            let num = num.clone();
-                            async move {
-                                rt.service("addr1", fn_service(|_| ok::<_, ()>(())));
-                                rt.service("addr3", fn_service(|_| ok::<_, ()>(())));
-                                rt.on_start(lazy(move |_| {
-                                    let _ = num.fetch_add(1, Relaxed);
-                                }));
-                                Ok::<_, io::Error>(())
-                            }
-                        })
-                })
-                .unwrap()
-                .workers(1)
-                .start()
-        });
-        let _ = tx.send((srv, ntex::rt::System::current()));
-        let _ = sys.run();
-    });
-    let (_, sys) = rx.recv().unwrap();
-    thread::sleep(time::Duration::from_millis(500));
-
-    assert!(net::TcpStream::connect(addr1).is_ok());
-    assert!(net::TcpStream::connect(addr2).is_ok());
-    assert!(net::TcpStream::connect(addr3).is_ok());
-    assert_eq!(num.load(Relaxed), 1);
-    sys.stop();
-    let _ = h.join();
-}
-
 #[test]
 fn test_on_worker_start() {
     let addr1 = TestServer::unused_addr();
@@ -243,8 +140,8 @@ fn test_on_worker_start() {
     let h = thread::spawn(move || {
         let num = num2.clone();
         let num2 = num2.clone();
-        let mut sys = ntex::rt::System::new("test");
-        let srv = sys.exec(|| {
+        let sys = ntex::rt::System::new("test");
+        let srv = sys.exec(move || {
             Server::build()
                 .disable_signals()
                 .configure(move |cfg| {
@@ -255,7 +152,7 @@ fn test_on_worker_start() {
                         .bind("addr2", addr2)
                         .unwrap()
                         .listen("addr3", lst)
-                        .apply_async(move |rt| {
+                        .on_worker_start(move |rt| {
                             let num = num.clone();
                             async move {
                                 rt.service("addr1", fn_service(|_| ok::<_, ()>(())));
@@ -273,7 +170,7 @@ fn test_on_worker_start() {
                     Ready::Ok::<_, io::Error>(())
                 })
                 .workers(1)
-                .start()
+                .run()
         });
         let _ = tx.send((srv, ntex::rt::System::current()));
         let _ = sys.run();
@@ -299,14 +196,14 @@ fn test_panic_in_worker() {
     let (tx, rx) = mpsc::channel();
 
     let h = thread::spawn(move || {
-        let mut sys = ntex::rt::System::new("test");
+        let sys = ntex::rt::System::new("test");
         let counter = counter2.clone();
         let srv = sys.exec(move || {
             let counter = counter.clone();
             Server::build()
                 .workers(1)
                 .disable_signals()
-                .bind("test", addr, move || {
+                .bind("test", addr, move |_| {
                     let counter = counter.clone();
                     fn_service(move |_| {
                         counter.fetch_add(1, Relaxed);
@@ -315,10 +212,10 @@ fn test_panic_in_worker() {
                     })
                 })
                 .unwrap()
-                .start()
+                .run()
         });
         let _ = tx.send((srv.clone(), ntex::rt::System::current()));
-        sys.exec(move || ntex_rt::spawn(srv.map(|_| ())));
+        sys.exec(move || ntex::rt::spawn(srv.map(|_| ())));
         let _ = sys.run();
     });
     let (_, sys) = rx.recv().unwrap();
